@@ -3,6 +3,8 @@ JPSLUtils = new Object();
 /*
 Initialization
 */
+JPSLUtils.env = "None";
+
 JPSLUtils.init = function(){
     // Run all input table cells to make sure the tables are showing and
     // active. Also hide the table creation code.
@@ -19,6 +21,38 @@ JPSLUtils.init = function(){
     JPSLUtils.hide_hide_code_code();
 };
 
+JPSLUtils.getenv = function(){
+    if (typeof (Jupyter) != 'undefined'){
+        JPSLUtils.env = "NBClassic";
+        Jupyter.notebook.kernel.execute('JPSLUtils.notebookenv = "'+JPSLUtils.env+'"');
+    } else {
+        var configscript = document.getElementById("jupyter-config-data");
+        if (configscript){
+            var config = JSON.parse(configscript.innerText);
+            var name = config['appName'];
+            if (name){
+                JPSLUtils.env = name;
+            }
+        }
+    }
+};
+
+/*
+Latex utilities
+*/
+
+JPSLUtils.pseudoLatexToLatex = function(text){
+    text = text.replaceAll('%FRAC','\\frac');
+    text = text.replaceAll('%EXP','\\exp');
+    text = text.replaceAll('%SIN','\\sin');
+    text = text.replaceAll('%LEFT','\\left');
+    text = text.replaceAll('%RIGHT','\\right');
+    text = text.replaceAll('%SQRT','\\sqrt');
+    text = text.replaceAll('%PI','\\pi');
+    text = text.replaceAll('%COLOR','\\color');
+    return(text);
+};
+
 /*
 Cell Utilities
 */
@@ -42,7 +76,21 @@ JPSLUtils.select_cell_immediately_below = function(){
     Jupyter.notebook.select_next(true);
 };
 
+JPSLUtils.text_of_current_cell_to_Python = function(varName){
+    var text = Jupyter.notebook.get_selected_cell().get_text();
+    if (typeof (text) == 'undefined'){
+        text = '';
+    }
+    var cmdstr = '"'+varName+' = \"'+ text +'\""';
+    //alert (cmdstr);
+    JPSLUtils.wait_for_python('\'' + cmdstr+'\'');
+        //.then(resolve => JPSLUtils.wait_for_python(
+        //'print(JPSLUtils.return_text_of_current_cell())');
+        //.then(reject => alert(reject));
+};
+
 JPSLUtils.replace_text_of_current_cell = function(text){
+    text = JPSLUtils.pseudoLatexToLatex(text);
     Jupyter.notebook.get_selected_cell().set_text(text);
 };
 
@@ -181,6 +229,21 @@ JPSLUtils.record_input = function (element){
 /*
 Python Execution
 */
+JPSLUtils.wait_for_python = function(cmdstr){
+    return new Promise((resolve,reject) => {
+        var callbacks = {
+            iopub: {
+                output: (data) => resolve(data.content.text.trim())
+            },
+
+            shell: {
+                reply: (data) => resolve(data.content.status)
+            }
+
+        };
+        Jupyter.notebook.kernel.execute(cmdstr, callbacks);
+    });
+}
 
 JPSLUtils.executePython = function(python) {
     return new Promise((resolve, reject) => {
@@ -215,7 +278,7 @@ JPSLUtils.record_names = function(){
     var tmp = document.createElement('H4');
     var inststr = "In the box below type your name and your partners' names";
     inststr += " (one per line):";
-    tmp.innerHTML=inststr
+    tmp.innerHTML=inststr;
     dlg.append(tmp);
     tmp = document.createElement('div');
     tmp.innerHTML = '<textarea cols="30" onblur="JPSLUtils.record_input(this)"/>';
@@ -242,7 +305,7 @@ JPSLUtils.record_names = function(){
                                     //tmpstr.replace(refeed,'; ');
                                     tmp.innerHTML = tmpstr;
                                     tmpstr = '# '+rcrd.innerHTML.replaceAll
-                                    ('|','\n# ') +'\n#  '+tmpstr;
+                                    ('|','\\n# ') +'\\n#  '+tmpstr;
                                     //rcrd.append(tmp);
                                     JPSLUtils.
                                     insert_newline_at_end_of_current_cell(
@@ -341,4 +404,6 @@ JPSLUtils.createJPSLToolsMenu = function(){
                     };
         JPSLMenus.build(menu);
     }
-}
\ No newline at end of file
+};
+
+JPSLUtils.getenv();
\ No newline at end of file
